package jp.co.celestee.ai.interfaces.image;

import java.awt.Color;
import java.awt.Point;
import java.awt.Polygon;
import java.util.ArrayList;
import java.util.List;

import jp.co.celestee.ai.core.signal.AbstractSignalData;
import jp.co.celestee.ai.core.signal.SignalDataIF;
import jp.co.celestee.ai.interfaces.SignalParts;
import jp.co.celestee.external.object.avatar.FloorColor;

/**
 * 摜̂̂̃f[^łB
 * @author celestee
 *
 */
public class ImageSignalData extends AbstractSignalData {

	private static final long serialVersionUID = 1L;

	/** FƂ\l */
	private static final Color COLOR_NULL = Color.WHITE;
	
	/** `Ƃ\l */
	private static final Polygon POLYGON_NULL = PolygonNull.getInstance();
	
	/** F */
	@SignalParts
	private Color color = null;
	
	/** ` */
	@SignalParts
	private Polygon polygon = null;
	
	public ImageSignalData() {
	}

	public ImageSignalData(Color color, Polygon polygon) {
		this.color = color;
		this.polygon = polygon;
	}
	
	@Override
	public Class<?> getOutputIdClass() {
		return ImageOutputId.class;
	}
	
	public Color getColor() {
		return color;
	}

	public void setColor(Color color) {
		this.color = color;
	}

	public Polygon getPolygon() {
		return polygon;
	}

	public void setPolygon(Polygon polygon) {
		this.polygon = polygon;
	}
	
	public static Color colorFreeValueReturn() {
		return COLOR_NULL;
	}

	public static Polygon polygonFreeValueReturn() {
		return POLYGON_NULL;
	}

	@Override
	public String toString() {
		String sColor = null;
		if(color == COLOR_NULL) {
			sColor = "NONE_COLOR";
		}
		else {
			sColor = color.toString();
		}
		
		return "(COLOR=" + sColor + " ,POLYGON=" + polygon + " ,In=" + isInner() + ")";
	}

	@Override
	public List<SignalDataIF> getPartsSignalDataList() {
		List<SignalDataIF> signalList = new ArrayList<SignalDataIF>(3);
		
		ImageSignalData dataC = new ImageSignalData(color, POLYGON_NULL);
		signalList.add(dataC);
		
		ImageSignalData dataP = new ImageSignalData(COLOR_NULL, polygon);
		signalList.add(dataP);
		
		return signalList;
	}
	
	@Override
	public boolean isChain(SignalDataIF targetSignalData) {
		if(targetSignalData instanceof ImageSignalData) {
			//TODO: {͔ėpIȎɂBiKł͓͂̕KɎĂB
			ImageSignalData data = (ImageSignalData)targetSignalData;
			if(getColor() instanceof FloorColor && data.getColor() instanceof FloorColor) {
				FloorColor color = (FloorColor)getColor();
				FloorColor targetColor = (FloorColor)data.getColor();
				Point point = color.getPoint();
				Point targetPoint = targetColor.getPoint();
				int x = Math.abs(point.x - targetPoint.x);
				int y = Math.abs(point.y - targetPoint.y);
				
				if(x + y <= 1) {
					return true;
				}
				
				//if(point.x == targetPoint.x || point.y == targetPoint.y) {
				//	return true;
				//}
				
			}
		}
		
		return false;
	}
	
	@Override
	public boolean equals(Object target) {
		boolean b = super.equals(target);
		if(b == false) return b;
		
		if(target instanceof ImageSignalData) {
			ImageSignalData data = (ImageSignalData)target;
			boolean bColor = color == null ? data.getColor() == null : color.equals(data.getColor());
			boolean bPolygon = polygon == null ?  data.getPolygon() == null : polygon.equals(data.getPolygon());
			return bColor && bPolygon;
		}
		
		return false;
	}
	
	@Override
	public int hashCode() {
		return 0;
	}

	@Override
	public String getSupposingItSaysInMansWords() {
		return "looking " + getColor() + " , " + getPolygon();
	}

}
